@@ -35,6 +35,7 @@
 #include "mainframewidget.h"
 #include "rest.h"
 #include "chord.h"
+#include "text.h"
 #include "chorddiagram.h"
 #include "layout.h"
 
@@ -601,6 +602,9 @@ bool NABCExport::writeFirstVoice(NVoice *voice_elem, QString staffName, int staf
 				     outputKeySig((NKeySig *) elem, false);
 				     out_ << ']';
 				     break;
+			case T_TEXT:
+				     voiceStatList_[idx].pendingText = (NText *) elem;
+				     break;
 			default:     inBeam =false;
 				     voiceStatList_[idx].lastBarSym = 0;
 				     if (inGrace) {
@@ -860,6 +864,10 @@ void NABCExport::writePendingSigns(int idx) {
 			case RITARDANDO: out_ << "!rit! "; break;
 		}
 	}
+	if (voiceStatList_[idx].pendingText) {
+		out_ << '"' << voiceStatList_[idx].pendingText->getText() << "\" ";
+		voiceStatList_[idx].pendingText = 0;
+	}
 	voiceStatList_[idx].pendingRitAccel = 0;
 }
 	
@@ -1236,7 +1244,6 @@ void NABCExport::outputMidi(QList<NStaff> *stafflist) {
 	NVoice *voice_elem;
 	int voice_count;
 	int i, j, k = 1;
-	extern char *instrTab[];
 	out_ << '%' << endl;
 
 	for (i = 0, staff_elem = stafflist->first(); staff_elem; i++, staff_elem = stafflist->next()) {
@@ -1244,7 +1251,7 @@ void NABCExport::outputMidi(QList<NStaff> *stafflist) {
 		for (j = 0; j < voice_count; j++) {
 			voice_elem = staff_elem->getVoiceNr(j);
 			out_ << "%%MIDI program " << k++ << ' ' << staff_elem->getVoice() << " % ";
-			out_ << i18n( instrTab[staff_elem->getVoice()] ) << endl;
+			out_ << i18n( NResource::instrTab[staff_elem->getVoice()] ) << endl;
 		}
 	}
 	out_ << '%' << endl;
