@@ -440,6 +440,14 @@ void NChord::setActualTied(bool tied) {
 	SET_STATUS(tied, note->status, STAT_TIED);
 }
 
+void NChord::removeAllTies(){
+	NNote *note;
+
+	for (note = noteList_.first(); note; note = noteList_.next()) {
+		note->status &= (~(STAT_TIED));
+	}
+}
+
 void NChord::tieWith(NChord *otherChordBefore) {
 	NNote *note, *note2;
 	if (status_ & STAT_GRACE) return;
@@ -1195,7 +1203,7 @@ void NChord::calculateDimensionsAndPixmaps() {
 	if (lyrics_) {
 		main_props_->directPainter->beginTextDrawing();
 		main_props_->directPainter->setPen(NResource::blackPen_);
-		main_props_->directPainter->setFont( *NResource::text_ );
+		main_props_->directPainter->setFont( *NResource::textFont_ );
 		for (i = 0; i < NUM_LYRICS; ++i) {
 		   if (lyrics_[i]) {
 			(*lyricsPoints_[i]) = QPoint (xpos_, staff_props_->base + 4 * LINE_DIST + i * LYRICS_Y_SPACE + staff_props_->lyricsdist);
@@ -2121,6 +2129,7 @@ QString *NChord::computeTeXBeam(int maxBeams, unsigned int *beamPool, int *beamN
 	return 0;
 }
 
+
 QString *NChord::computeTeXTie(unsigned int *tiePool, NClef *clef, int maxtie, bool *toomany, bool spare) {
 	int tieNr = 0;
 	char tieDir;
