@@ -38,6 +38,7 @@
 #include "outputbox.h"
 #include "parsertypes.h"
 #include "staff.h"
+#include "text.h"
 #include "uiconnect.h"
 #include "chorddiagram.h"
 #include "layout.h"
@@ -71,6 +72,8 @@ NFileHandler::NFileHandler() : newLines_("\n") {
 	rolls_.truncate(0);
 	pedals_.truncate(0);
 	signs_.truncate(0);
+	textsigns_above_.truncate(0);
+	textsigns_below_.truncate(0);
 	badlist_.setAutoDelete(true);
 	fatallist_.setAutoDelete(true);
 	pending_volume_sigs_.setAutoDelete(true);
@@ -348,6 +351,8 @@ bool NFileHandler::writeStaffs(QString fname, QList<NStaff> *stafflist, NMainFra
 			rolls_.truncate(0);
 			pedals_.truncate(0);
 			signs_.truncate(0);
+			textsigns_above_.truncate(0);
+			textsigns_below_.truncate(0);
 			lyricsLineCount_ = 0;
 			pending_volume_sigs_.clear();
 			pending_tempo_sigs_.clear();
@@ -431,6 +436,12 @@ bool NFileHandler::writeStaffs(QString fname, QList<NStaff> *stafflist, NMainFra
 			if (!pedals_.isEmpty()) {
 				out_ << pedals_ << endl;
 			}
+			if (!textsigns_above_.isEmpty()) {
+				out_ << textsigns_above_ << endl;
+			}
+			if (!textsigns_below_.isEmpty()) {
+				out_ << textsigns_below_ << endl;
+			}
 			if (!signs_.isEmpty()) {
 				if (!some_notes_or_rests_written_) {
 						bad = new badmeasure(ERR_SEGNOS_AT_END, i, bar_nr_, 3 /* dummy */, countof128th_);
@@ -757,6 +768,7 @@ int NFileHandler::writeStaffUntilBar(int staff_nr, NVoice *voi, bool first, int
 	NSign *sign;
 	NClef *clef;
 	NKeySig *ksig;
+	NText *textSign;
 	bool ok;
 	int tupletsum;
 	int subtype;
@@ -823,6 +835,26 @@ int NFileHandler::writeStaffUntilBar(int staff_nr, NVoice *voi, bool first, int
 					}
 				 	elem = voi->getNextPosition();
 					break;
+				case T_TEXT:
+					textSign = (NText *) elem;
+					if (textSign->getSubType() == TEXT_UPTEXT) {
+						if (textsigns_above_.isEmpty()) {
+							textsigns_above_.sprintf("rom (14) above %d: ",  staff_nr);
+						}
+						timestring = "1 \"";
+						timestring += textSign->getText() + "\";";
+						textsigns_above_ += timestring;
+					}
+					else {
+						if (textsigns_below_.isEmpty()) {
+							textsigns_below_.sprintf("rom (14) below %d: ",  staff_nr);
+						}
+						timestring = "1 \"";
+						timestring += textSign->getText() + "\";";
+						textsigns_below_ += timestring;
+					}
+				 	elem = voi->getNextPosition();
+					break;
 				default: elem = voi->getNextPosition();
 					break;
 			}
@@ -1214,7 +1246,7 @@ int NFileHandler::writeStaffUntilBar(int staff_nr, NVoice *voi, bool first, int
 				     }
 				     out_ << ((elem->status_ & STAT_HIDDEN) ? "s; " : "r; ");
 				     if ((diag = elem->getChordChordDiagram())) {
-					starttime = ((elem->midiTime_ - *measure_start_time) / 3) / (double) (128 / curr_denom_);
+					starttime = ((elem->midiTime_ - *measure_start_time) / MULTIPLICATOR) / (double) (128 / curr_denom_);
 					writeChord(staff_nr, starttime, diag);
 				     }
 				     if (elem->status_ & STAT_LAST_TUPLET) {
@@ -1352,6 +1384,32 @@ int NFileHandler::writeStaffUntilBar(int staff_nr, NVoice *voi, bool first, int
 				bad = new badmeasure(ERR_KEYCHANGE, staff_nr, bar_nr_, 3 /* dummy */, countof128th_);
 				badlist_.append(bad);
 				break;
+			case T_TEXT:
+				if (inGrace) {
+					inGrace = false;
+					beam_grace_problem_reported = false;
+					bad = new badmeasure(ERR_GRACE_AFTER, staff_nr, bar_nr_, total / 3, countof128th_);
+					badlist_.append(bad);
+				}
+				starttime = ((elem->midiTime_ - *measure_start_time) / MULTIPLICATOR) / (double) (128 / curr_denom_);
+				textSign = (NText *) elem;
+				if (textSign->getSubType() == TEXT_UPTEXT) {
+					if (textsigns_above_.isEmpty()) {
+						textsigns_above_.sprintf("rom (14) above %d: ",  staff_nr);
+					}
+					timestring.sprintf("%f \"", starttime + 1.0);
+					timestring += textSign->getText() + "\";";
+					textsigns_above_ += timestring;
+				}
+				else {
+					if (textsigns_below_.isEmpty()) {
+						textsigns_below_.sprintf("rom (14) below %d: ",  staff_nr);
+					}
+					timestring.sprintf("%f \"", starttime + 1.0);
+					timestring += textSign->getText() + "\";";
+					textsigns_below_ += timestring;
+				}
+				break;
 		}
 		elem = voi->getNextPosition();
 	}
