@@ -48,6 +48,7 @@
 #include "timesig.h"
 #include "sign.h"
 #include "rest.h"
+#include "text.h"
 #include "chord.h"
 #include "voice.h"
 #include "staff.h"
@@ -147,13 +148,26 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 	out_ << "%                                  %" << endl;
 	out_ << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl << endl;
 	if (!exportDialog_->lilyVoice->isChecked()) {
-		out_ << "\\include \"paper" << exportDialog_->lilyFont->currentText() << ".ly\"" << endl << endl;
+		if (NResource::lilyProperties_.lilyProperties) {
+			out_ << "\\include \"paper" << exportDialog_->lilyFont->currentText() << ".ly\"" << endl << endl;
+		}
+		else {
+			out_ << "#(set-global-staff-size " << exportDialog_->lilyFont->currentText() <<')' << endl << endl;
+		}
 	}
 	if (exportDialog_->lilyDrumNotes->isChecked()) {
-		out_ << "bcr = {" << endl << "\\context Thread = xcircle" << endl << "\\property Voice.NoteHead \\set #'style = #'cross" << endl << '}' << endl;
-		out_ << "bdf = {" << endl << "\\context Thread = xdefault" << endl << "\\property Voice.NoteHead \\set #'style = #'default" << endl << '}' << endl;
-		out_ << "bcc = {" << endl << "\\context Thread = xcircle" << endl << "\\property Voice.NoteHead \\set #'style = #'xcircle" << endl << '}' << endl;
-		out_ << "btr = {" << endl << "\\context Thread = xtriangle" << endl << "\\property Voice.NoteHead \\set #'style = #'triangle" << endl << '}' << endl << endl;
+		if (NResource::lilyProperties_.lilyProperties) {
+			out_ << "bcr = {" << endl << "\\context Thread = xcircle" << endl << "\\property Voice.NoteHead \\set #'style = #'cross" << endl << '}' << endl;
+			out_ << "bdf = {" << endl << "\\context Thread = xdefault" << endl << "\\property Voice.NoteHead \\set #'style = #'default" << endl << '}' << endl;
+			out_ << "bcc = {" << endl << "\\context Thread = xcircle" << endl << "\\property Voice.NoteHead \\set #'style = #'xcircle" << endl << '}' << endl;
+			out_ << "btr = {" << endl << "\\context Thread = xtriangle" << endl << "\\property Voice.NoteHead \\set #'style = #'triangle" << endl << '}' << endl << endl;
+		}
+		else {
+			out_ << "bcr = {" << endl << "\\context Thread = xcircle" << endl << "\\override Voice.NoteHead #'style = #'cross" << endl << '}' << endl;
+			out_ << "bdf = {" << endl << "\\context Thread = xdefault" << endl << "\\override Voice.NoteHead #'style = #'default" << endl << '}' << endl;
+			out_ << "bcc = {" << endl << "\\context Thread = xcircle" << endl << "\\override Voice.NoteHead #'style = #'xcircle" << endl << '}' << endl;
+			out_ << "btr = {" << endl << "\\context Thread = xtriangle" << endl << "\\override Voice.NoteHead #'style = #'triangle" << endl << '}' << endl << endl;
+		}
 	}
 	if (NResource::lilyProperties_.lilyVersion2) {
 		out_ << "flat = \\markup { \\raise #0.4 \\smaller \\musicglyph #\"accidentals--2\" }" << endl;
@@ -225,13 +239,28 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 			for (k = 0; k < str; k++) out_ << '\'';
 			out_ << " {" << endl << '\t';
 			if (exportDialog_->lilyBeam->isChecked()) {
-				out_ << "\\property Voice.autoBeaming = ##f" << endl << '\t';
+				if (NResource::lilyProperties_.lilyProperties) {
+					out_ << "\\property Voice.autoBeaming = ##f" << endl << '\t';
+				}
+				else {
+					out_ << "\\set Voice.autoBeaming = ##f" << endl << '\t';
+				}
 			}
 			if (exportDialog_->lilyTies->isChecked()) {
-				out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
+				if (NResource::lilyProperties_.lilyProperties) {
+					out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
+				}
+				else {
+					out_ << "\\set Thread.sparseTies = ##t" << endl << '\t';
+				}
 			}
 			if (!voice_elem->getStaff()->staffName_.isNull() && !voice_elem->getStaff()->staffName_.isEmpty()) {
-				out_ << "\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl << '\t';
+				if (NResource::lilyProperties_.lilyProperties) {
+					out_ << "\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl << '\t';
+				}
+				else {
+					out_ << "\\set Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl << '\t';
+				}
 			}
 			voice_elem->prepareForWriting();
 			staff_elem->actualClef_.change(clef);
@@ -273,10 +302,20 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 				}
 				out_ <<  endl << '\t';
 				if (exportDialog_->lilyBeam->isChecked()) {
-					out_ << "\\property Voice.noAutoBeaming = ##t" << endl << '\t';
+					if (NResource::lilyProperties_.lilyProperties) {
+						out_ << "\\property Voice.noAutoBeaming = ##t" << endl << '\t';
+					}
+					else {
+						out_ << "\\set Voice.noAutoBeaming = ##t" << endl << '\t';
+					}
 				}
 				if (exportDialog_->lilyTies->isChecked()) {
-					out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
+					if (NResource::lilyProperties_.lilyProperties) {
+						out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
+					}
+					else {
+						out_ << "\\set Thread.sparseTies = ##t" << endl << '\t';
+					}
 				}
 				voice_elem->prepareForWriting();
 				staff_elem->actualClef_.change(clef);
@@ -294,7 +333,12 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 			removeExceptsFromString(&staffLabel, false);
 			out_ << staffLabel << " = \\simultaneous {" << endl;
 			if (!staff_elem->staffName_.isNull() && !staff_elem->staffName_.isEmpty()) {
-				out_ << "\t\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl;
+				if (NResource::lilyProperties_.lilyProperties) {
+					out_ << "\t\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl;
+				}
+				else {
+					out_ << "\t\\set Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl;
+				}
 			}
 			for (j = 0, voice_elem = staff_elem->getVoiceNr(0); j < voice_count; j++) {
 				if (staff_elem->staffName_.isNull() || staff_elem->staffName_.isEmpty()) {
@@ -323,9 +367,19 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 	if (!exportDialog_->lilyVoice->isChecked()) {
 		out_ << "\\score {" << endl;
 		out_ << "\t\\simultaneous {" << endl;
-		out_ << "\t\t\\property Score.skipBars = ##t" << endl;
+		if (NResource::lilyProperties_.lilyProperties) {
+			out_ << "\t\t\\property Score.skipBars = ##t" << endl;
+		}
+		else {
+			out_ << "\t\t\\set Score.skipBars = ##t" << endl;
+		}
 		if (NResource::lilyProperties_.lilyVersion2) {
-			out_ << "\t\t\\property Score.melismaBusyProperties = #'(melismaBusy slurMelismaBusy tieMelismaBusy)" << endl;
+			if (NResource::lilyProperties_.lilyProperties) {
+				out_ << "\t\t\\property Score.melismaBusyProperties = #'(melismaBusy slurMelismaBusy tieMelismaBusy)" << endl;
+			}
+			else {
+				out_ << "\t\t\\set Score.melismaBusyProperties = #'(melismaBusy slurMelismaBusy tieMelismaBusy)" << endl;
+			}
 		}
 		depth_ = 2;
 	
@@ -584,6 +638,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 	int c4Line;
 	int restlen;
 	int octaviation = 0;
+	NText *pending_text = 0;
 
 	countof128th_ = 128;
 	currentNumerator_ = currentDenominator_ = 4;
@@ -661,7 +716,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 						out_ << "\\fatText ";
 					}
 				     }
-				     if ((pending_segnos_rirads_accels & SOME_SIGNS_WITH_TEXT) && !fatTextWritten) {
+				     if ((pending_text || (pending_segnos_rirads_accels & SOME_SIGNS_WITH_TEXT)) && !fatTextWritten) {
 				     	fatTextWritten = true;
 					out_ << "\\fatText ";
 				     }
@@ -673,20 +728,35 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 								textpropset = true;
 								out_ << endl;
 								tabsOut();
-								out_ << "\\property Voice.TextSpanner \\set #'type = #'trill" << endl;
+								if (NResource::lilyProperties_.lilyProperties) {
+									out_ << "\\property Voice.TextSpanner \\set #'type = #'trill" << endl;
+								}
+								else {
+									out_ << "\\override Voice.TextSpanner #'type = #'trill" << endl;
+								}
 							}
 							if (chord->trill_ > 0 && edgetext != EDGE_TEXT_TR) {
 								edgetext = EDGE_TEXT_TR;
 								out_ << endl;
 								tabsOut();
-								out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'((line (music \"scripts-trill\") \" \") . \"\")" << endl;
+								if (NResource::lilyProperties_.lilyProperties) {
+									out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'((line (music \"scripts-trill\") \" \") . \"\")" << endl;
+								}
+								else {
+									out_ << "\\override Voice.TextSpanner #'edge-text = #'((line (music \"scripts-trill\") \" \") . \"\")" << endl;
+								}
 								tabsOut();
 							}
 							else if (chord->trill_ < 0 && edgetext != EDGE_TEXT_EMPTY) {
 								edgetext = EDGE_TEXT_EMPTY;
 								out_ << endl;
 								tabsOut();
-								out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'(\"\")" << endl;
+								if (NResource::lilyProperties_.lilyProperties) {
+									out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'(\"\")" << endl;
+								}
+								else {
+									out_ << "\\override Voice.TextSpanner #'edge-text = #'(\"\")" << endl;
+								}
 								tabsOut();
 							}
 						}
@@ -915,6 +985,16 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 								}
 								pendingVolSig = 0;
 							}
+							if (pending_text) {
+								if (pending_text->getSubType() == TEXT_UPTEXT) {
+									out_ << "^#'(large \"" << pending_text->getText() << "\") ";
+								}
+								else {
+									out_ << "_#'(large \"" << pending_text->getText() << "\") ";
+								}
+								pending_text = 0;
+							}
+									
 							if (pending_tempo >= 0) {
 						   		out_ << "^#`(columns , quarter-head \" = " << pending_tempo << "\") ";
 						   		pending_tempo = -1;
@@ -951,11 +1031,11 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 								out_ << "^#'(large \"accel.\") ";
 								pending_segnos_rirads_accels &= (~(PENDING_ACCELERANDO));
 							}
+							if (chord->status_ & STAT_SLURED) {
+						   		out_ << " ( ";
+							}
 						}
 						saveLine = lastLine_;
-						if (chord->status_ & STAT_SLURED) {
-						   out_ << " ( ";
-						}
 					     }
 				     	     out_ << ' ';
 				     }
@@ -980,6 +1060,9 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					        }
 				     }
 				     if (NResource::lilyProperties_.lilyVersion2) {
+					if (chord->status_ & STAT_SLURED) {
+					  		out_ << " ( ";
+					}
 					if (pendingVolSig) {
 						switch (exportDialog_->lilyVol->currentItem()) {
 							case 1: out_ << '^' << pendingVolSig; break;
@@ -987,6 +1070,15 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 						}
 						pendingVolSig = 0;
 					}
+					if (pending_text) {
+						if (pending_text->getSubType() == TEXT_UPTEXT) {
+							out_ << "^\\markup{\\large\\bold " << pending_text->getText() << "} ";
+						}
+						else  {
+							out_ << "_\\markup{\\large\\bold " << pending_text->getText() << "} ";
+						}
+						pending_text = 0;
+					}
 					if (elem->status_ & STAT_ARPEGG) {
 						out_ << "\\arpeggio ";
 					}
@@ -1588,6 +1680,10 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 						break;
 				     }
 				     break;
+			case T_TEXT:
+				     pending_text = (NText *) elem;
+				     break;
+				    
 		}
 		elem = voi->getNextPosition();
 	}
