@@ -311,6 +311,7 @@ class NMainFrameWidget : public QWidget
 		void transposeDialog();
 		void voiceChangeDialog();
 		void insertLine();
+		void insertText();
 		void clefDialog();
 		void changeClefDialog();
 		void redAccidentals();
@@ -558,6 +559,7 @@ class NMainFrameWidget : public QWidget
 		void TSE3MidiOut();
 		void TSE3MidiIn();
 		void TSE3toScore();
+		void TSE3ParttoScore();
 		void TSE3Filter();
 		void completeRecording(bool);
 		void completeTSE3toScore(bool ok);
