@@ -41,20 +41,37 @@
 
 #define TIM(x) x / M3, x / TRI16, x
 
-NMidiTimeScale::NMidiTimeScale() {
+NMidiTimeScale::NMidiTimeScale(int noteSnap, int smallestTupletNote) : noteSnap_(noteSnap), smallestTupletNote_(smallestTupletNote) {
 	array_len_ = 0;
 	alloc_len_ = 0;
 	unrolled_midi_events_ = 0;
 	midi_program_ = -1;
+	time_array_ = 0;
+	time_array_len_ = 0;
+	time_array_alloc_len_ = 0;
+	switch (smallestTupletNote) {
+		case HALF_LENGTH: mink_ = 8; break;
+		case QUARTER_LENGTH: mink_ = 4; break;
+		case NOTE8_LENGTH: mink_ = 2; break;
+		case -2:
+		case NOTE16_LENGTH: mink_ = 1; break;
+		default: mink_ = 1; break;
+	}
 }
 
 NMidiTimeScale::~NMidiTimeScale() {
 	if (unrolled_midi_events_) {
 		free(unrolled_midi_events_);
 	}
+	if (time_array_) {
+		free(time_array_);
+	}
 	unrolled_midi_events_ = 0;
 	array_len_ = 0;
 	alloc_len_ = 0;
+	time_array_ = 0;
+	time_array_len_ = 0;
+	time_array_alloc_len_ = 0;
 }
 
 int NMidiTimeScale::getMidiProgram() {
@@ -73,16 +90,20 @@ void NMidiTimeScale::removeEvent(unsigned int idx) {
 }
 
 void NMidiTimeScale::insertEvent(struct unrolled_midi_events_str *ptr) {
-	unsigned int i, j, k, l;
+	unsigned int i, j, k, l, len, len1;
 	unrolled_midi_events_str *ptr1;
 	bool inserted, equal;
 
 	l = array_len_;
+	len = ptr->stop_time - ptr->start_time;
 	for (i = 0, ptr1 = unrolled_midi_events_; i < l && ptr1->start_time < ptr->start_time; i++, ptr1++);
 	inserted = false;
 	if (i < l && (ptr1->eventType & EVT_CLASS_REST) == 0 && (ptr1->eventType & EVT_NORMAL_EVENT) && (ptr->eventType & EVT_NORMAL_EVENT)) {
+		len1 = ptr->stop_time - ptr->start_time;
 		for (;!inserted && i < l && ptr1->start_time == ptr->start_time; i++, ptr1++) {
-			if (ptr1->stop_time == ptr->stop_time) {
+			if (ptr1->stop_time >= ptr->stop_time - len / 2 && ptr1->stop_time <= ptr->stop_time + len / 2 ||
+				ptr->stop_time >= ptr1->stop_time - len1 / 2 && ptr->stop_time <= ptr1->stop_time + len1 / 2
+) {
 				inserted = true;
 				equal = false; // Rosegarden produces some MIDI notes n times (?)
 				for (k = 0; !equal && k < ptr1->U.norm_evt.num_pitches; k++) {
@@ -127,7 +148,7 @@ void NMidiTimeScale::insertEvent(struct unrolled_midi_events_str *ptr) {
 int NMidiTimeScale::determine_snap(int length) {
 	int compare;
 
-	compare = DOUBLE_WHOLE_LENGTH;
+	compare = NOTE8_LENGTH;
 
 	do {
 		if (compare < length) {
@@ -140,7 +161,44 @@ int NMidiTimeScale::determine_snap(int length) {
 	return (NOTE64_LENGTH >> 1);
 }
 
-#ifdef WITH_TRIPLET_RECOGNITION
+void NMidiTimeScale::insertTimeOfTimesig(unsigned int timeSigtime) {
+	unsigned int i, j;
+	unsigned int *iptr;
+
+	for (i = 0, iptr = time_array_; i < time_array_len_ && *iptr < timeSigtime; i++, iptr++);
+	if (time_array_len_ >= time_array_alloc_len_) {
+		if (time_array_) {
+			time_array_alloc_len_ += SEGMENT_LEN;
+			if ((time_array_ = (unsigned int *) realloc(time_array_, time_array_alloc_len_ * sizeof(unsigned int))) == NULL) {
+					NResource::abort("NMidiTimeScale::insertTimeOfTimesig", 1);
+			}
+		}
+		else {
+			time_array_alloc_len_ = SEGMENT_LEN;
+			if ((time_array_ = (unsigned int *) malloc(time_array_alloc_len_ * sizeof(unsigned int))) == NULL) {
+					NResource::abort("NMidiTimeScale::insertTimeOfTimesig", 2);
+			}
+		}
+	}
+	for (j = time_array_len_; j > i; j--) {
+		time_array_[j] = time_array_[j-1];
+	}
+	time_array_[i] = timeSigtime;
+	time_array_len_++;
+}
+
+unsigned int NMidiTimeScale::lastTimeSigTime(unsigned int currentTime) {
+	unsigned int i;
+	unsigned int *iptr, *last_iptr = 0;
+
+	for (i = 0, iptr = time_array_; i < time_array_len_ && *iptr < currentTime; last_iptr = iptr, i++, iptr++);
+	if (last_iptr) {
+		return *last_iptr;
+	}
+	return 0;
+}
+
+
 int NMidiTimeScale::quantTriplet(int l, struct unrolled_midi_events_str *ptr, bool *addRest, int maxlength) {
 	unsigned int testlength;
 	unsigned int deltamin1 = (1<<30);
@@ -169,7 +227,6 @@ int NMidiTimeScale::quantTriplet(int l, struct unrolled_midi_events_str *ptr, bo
 	return (MULTIPLICATOR << i);
 }
 
-#endif
 
 int NMidiTimeScale::quantNote(int l, int *dotcount, int maxlength) {
 	unsigned int testlength;
@@ -423,12 +480,17 @@ void NMidiTimeScale::findPathsInChunk(unsigned int chunkStartIdx, unsigned int c
 	unrolled_midi_events_str *ptr, *ptr2;
 	unsigned int len = chunkEndIdx - chunkStartIdx;
 	unsigned int i, k;
+	int j, max_ave_idx, tmp;
 	int pitch_sum;
 	bool endof_path;
+	int num_pitches;
+	int voice_map[MAX_VOICES];
+	double average_pitches[MAX_VOICES], max_pitch;
 	int fidx, lidx, pidx, len2;
 	struct path_elem_str *shortest_path;
 	int voice_nr = 0;
 
+
 	len2 = len;
 
 	for (i = chunkStartIdx, ptr = &(unrolled_midi_events_[chunkStartIdx]); i <= chunkEndIdx; i++, ptr++) {
@@ -455,10 +517,14 @@ void NMidiTimeScale::findPathsInChunk(unsigned int chunkStartIdx, unsigned int c
 	}
 	while ((fidx = findFirstUclassified(chunkStartIdx, len + 1)) >= 0) {
 		if (voice_nr >= MAX_VOICES) {
-			NResource::abort("NMidiTimeScale::findPathsInChunk", 1);
+			fprintf(stderr, "too many voices, notes omitted\n"); fflush(stderr);
+			//NResource::abort("NMidiTimeScale::findPathsInChunk", 1);
 		}
+		num_pitches = 0;
+		if (voice_nr < MAX_VOICES) average_pitches[voice_nr] = 0.0;
 		if (len == 0) {
 			unrolled_midi_events_[fidx].voice_nr = voice_nr;
+			if (voice_nr < MAX_VOICES) average_pitches[voice_nr] = unrolled_midi_events_[fidx].ave_pitch;
 			voice_nr++;
 			continue;
 		}
@@ -476,6 +542,8 @@ void NMidiTimeScale::findPathsInChunk(unsigned int chunkStartIdx, unsigned int c
 				 NResource::abort("NMidiTimeScale::findPathsInChunk", 4);
 			}
 			unrolled_midi_events_[lidx].voice_nr = voice_nr;
+			if (voice_nr < MAX_VOICES) average_pitches[voice_nr] += unrolled_midi_events_[fidx].ave_pitch;
+			num_pitches++;
 			if (lidx == fidx) {
 				endof_path = true;
 			}
@@ -496,9 +564,43 @@ void NMidiTimeScale::findPathsInChunk(unsigned int chunkStartIdx, unsigned int c
 				}
 			}
 		}
+#define NUMBER_OF_NOTES_FAC 0.5
+		if (voice_nr < MAX_VOICES) {
+			average_pitches[voice_nr] /= num_pitches;
+			average_pitches[voice_nr] += NUMBER_OF_NOTES_FAC * num_pitches;
+		}
 		voice_nr++;
 	}
-	if (voice_nr > max_voices_) max_voices_ = voice_nr;
+	if (voice_nr > max_voices_ && voice_nr < MAX_VOICES) max_voices_ = voice_nr;
+
+	if (voice_nr < MAX_VOICES) {
+		for (i = 0; i < voice_nr; i++) {
+			voice_map[i] = i;
+	
+		}
+	
+		for (i = 0; i < voice_nr - 1; i++) {
+			max_pitch = average_pitches[i];
+			max_ave_idx = i;
+			for (j = i + 1; j < voice_nr; j++) {
+				if (average_pitches[j] > max_pitch) {
+					max_pitch = average_pitches[j];
+					max_ave_idx = j;
+				}
+			}
+			if (max_ave_idx != i) {
+				tmp = voice_map[i];
+				voice_map[i] = voice_map[max_ave_idx];
+				voice_map[max_ave_idx] = tmp;
+			}
+		}
+	}
+
+	for (i = chunkStartIdx, ptr = &(unrolled_midi_events_[chunkStartIdx]); i <= chunkEndIdx; i++, ptr++) {
+		if ((ptr->eventType & (EVT_NORMAL_EVENT | EVT_PSEUDO_TRIPLET_NOTE)) == 0) continue;
+		ptr->voice_nr = voice_map[ptr->voice_nr];
+		ptr->stem_policy = (voice_nr == 1) ? STEM_POL_INDIVIDUAL : ((ptr->voice_nr < voice_nr / 2) ? STEM_POL_UP : STEM_POL_DOWN);
+	}
 }
 
 void NMidiTimeScale::initialize_desicion_tree(unrolled_midi_events_str *ptr, unsigned int idx, unsigned int chunkStartIdx, unsigned int chunkEndIdx, int len) {
@@ -530,11 +632,10 @@ void NMidiTimeScale::findVoices() {
 	unsigned int chunkStartIdx = 0, chunkEndIdx;
 	bool chunkOk, removed, inserted;
 	unrolled_midi_events_str *ptr, *ptr2, *templist[TLIST_MAX];
-	unsigned int i, l, vnr;
+	unsigned int i, l, vnr, st_policy;
 	int j;
 	max_voices_ = 0;
 
-#ifdef WITH_TRIPLET_RECOGNITION
 
 	do { // remove all triplet members there are copies of this events in EVT_PSEUDO_TRIPLET_NOTE
 		removed = false;
@@ -546,7 +647,6 @@ void NMidiTimeScale::findVoices() {
 		}
 	}
 	while (removed);
-#endif
 	do {
 		chunkEndIdx = findNextChunkEnd(&chunkOk, &chunkStartIdx);
 		if (chunkOk) {
@@ -568,10 +668,9 @@ void NMidiTimeScale::findVoices() {
 	}
 	while (chunkOk);
 
-#ifdef WITH_TRIPLET_RECOGNITION
 
 	do { // insert the copies of the triplet members in EVT_PSEUDO_TRIPLET_NOTE again, this is necessary because of
-	     // the array shifts, neither the indox nor the pointer to an event can act as
+	     // the array shifts, neither the index nor the pointer to an event can act as
 	     // identificator because the array insert and remove copies the array members
 		inserted = false;
 		for (i = 0; i < array_len_; i++) {
@@ -580,11 +679,13 @@ void NMidiTimeScale::findVoices() {
 			if (ptr->U.pseudo_evt.members_inserted_again) continue;
 			l = ptr->U.pseudo_evt.num_triplet_members;
 			vnr = ptr->voice_nr;
+			st_policy = ptr->stem_policy;
 			ptr->U.pseudo_evt.members_inserted_again = true;
 			memcpy(templist, ptr->U.pseudo_evt.triplet_members, l * sizeof(struct unrolled_midi_events_str *));
 			for (j = 0; j < l; j++) { // temp vars because "insertEvent" shifts the array
 				ptr2 = templist[j];
 				ptr2->voice_nr = vnr;
+				ptr2->stem_policy = st_policy;
 				insertEvent(ptr2); // makes a copy
 				free(ptr2);
 			}
@@ -592,43 +693,12 @@ void NMidiTimeScale::findVoices() {
 		}
 	}
 	while (inserted);
-
-#endif
 }
 
-#ifdef WITH_TRIPLET_RECOGNITION
+#define COST_FOR_REST 100
+#define COSTS_FOR_SPLIT 0
+#define HIGH_COSTS 10000
 
-int NMidiTimeScale::search_for_event(int start_type, unsigned int start_time, int stop_type, unsigned int stop_time) {
-	unrolled_midi_events_str *ptr;
-	unsigned int i;
-	bool ok;
-
-	for (i = 0, ptr = &(unrolled_midi_events_[0]); i < array_len_; i++, ptr++) {
-		if ((ptr->eventType & EVT_NORMAL_EVENT) == 0) continue;
-		ok = false;
-		switch (start_type) {
-			case TRIPLET_SNAP: if (ptr->U.norm_evt.triplet_start_time > start_time + WHOLE_LENGTH) return -1;
-					   if (ptr->U.norm_evt.sta2diff <= ptr->U.norm_evt.sta3diff) break;
-					   if (ptr->U.norm_evt.triplet_start_time == start_time) ok = true;
-					   break;
-			case SYSTEM2_SNAP: if (ptr->start_time > start_time) return -1;
-					   if (ptr->start_time  == start_time) ok = true;
-					   break;
-			default: NResource::abort("NMidiTimeScale::search_for_event", 1);
-		}
-		if (!ok) continue;
-		switch (stop_type) {
-			case TRIPLET_SNAP: 
-					   if (ptr->U.norm_evt.sto2diff <= ptr->U.norm_evt.sto3diff) break;
-					   if (ptr->U.norm_evt.triplet_stop_time == stop_time) return i;
-					   break;
-			case SYSTEM2_SNAP: if (ptr->stop_time  == stop_time) return i;
-					   break;
-			default: NResource::abort("NMidiTimeScale::search_for_event", 2);
-		}
-	}
-	return -1;
-}
 
 int NMidiTimeScale::search_for_event_enlarge(int start_type, unsigned int start_time, int stop_type, unsigned int stop_time) {
 	unrolled_midi_events_str *ptr;
@@ -655,8 +725,8 @@ int NMidiTimeScale::search_for_event_enlarge(int start_type, unsigned int start_
 					if (diff < 0) diff = -diff;
 #ifdef XXX
 					   if (ok) {
-					   	printf("diff = %d(%d/%d), stop_time = %d(%d/%d), raw = %d(%d/%d), ptr->U.norm_evt.triplet_stop_time = %d(%d/%d)\n",
-						TIM(diff), TIM(stop_time), TIM(ptr->U.norm_evt.rawlen), TIM(ptr->U.norm_evt.triplet_stop_time)); fflush(stdout);
+					   	printf("diff = %d(%d/%d), stop_time = %d(%d/%d), ptr->U.norm_evt.triplet_stop_time = %d(%d/%d)\n",
+						TIM(diff), TIM(stop_time), TIM(ptr->U.norm_evt.triplet_stop_time)); fflush(stdout);
 					   }
 #endif
 					   if (diff <= (stop_time - start_time) / 4) {
@@ -678,183 +748,228 @@ int NMidiTimeScale::search_for_event_enlarge(int start_type, unsigned int start_
 	return -1;
 }
 
-void NMidiTimeScale::addTRest(unsigned int start_time, unsigned int stop_time) {
-	int i;
+int NMidiTimeScale::search_cuttable_note_left(unsigned int split_time, unsigned int stop_time) {
+	unrolled_midi_events_str *ptr;
+	unsigned int i;
+	int diff;
+
+	for (i = 0, ptr = &(unrolled_midi_events_[0]); i < array_len_; i++, ptr++) {
+		if ((ptr->eventType & EVT_NORMAL_EVENT) == 0) continue;
+		if (ptr->start_time > split_time) return -1;
+		diff = stop_time - ptr->U.norm_evt.triplet_stop_time;
+		if (diff < 0) diff = -diff;
+		if (diff <= (stop_time - split_time) / 4) {
+			ptr->U.norm_evt.triplet_stop_time = stop_time;
+			return i;
+		}
+	}
+	return -1;
+}
+
+int NMidiTimeScale::search_cuttable_note_right(unsigned int split_time, unsigned int start_time) {
+	unrolled_midi_events_str *ptr;
+	unsigned int i;
+	int diff;
+
+	for (i = 0, ptr = &(unrolled_midi_events_[0]); i < array_len_; i++, ptr++) {
+		if ((ptr->eventType & EVT_NORMAL_EVENT) == 0) continue;
+		if (ptr->start_time > split_time) return -1;
+		if (ptr->stop_time <= split_time) continue;
+		if (!is_nearby(ptr->U.norm_evt.triplet_start_time, start_time, ptr->stop_time - ptr->U.norm_evt.triplet_start_time)) continue;
+		ptr->U.norm_evt.triplet_start_time = start_time;
+		return i;
+	}
+	return -1;
+}
+
+
+void NMidiTimeScale::appendTRest(unsigned int start_time, unsigned int stop_time) {
 
-	if (TListPtr_ < 0) return;
 	if (TListPtr_ >= TLIST_MAX) {
-		NResource::abort("TLIST"); 
+		NResource::abort("appendTRest"); 
 	}
 	TList_[TListPtr_  ].evt_class = EVT_CLASS_REST;
 	TList_[TListPtr_  ].start_time = start_time;
 	TList_[TListPtr_++].stop_time = stop_time;
 }
-	
 
-void NMidiTimeScale::addTidx(int idx) {
+void NMidiTimeScale::prependTRest(unsigned int start_time, unsigned int stop_time) {
 	int i;
 
-	if (TListPtr_ < 0) return;
 	if (TListPtr_ >= TLIST_MAX) {
-		NResource::abort("TLIST"); 
+		NResource::abort("prependTRest"); 
 	}
-	for (i = 0; i < TListPtr_; i++) {
-		if (TList_[i].evt_class == EVT_CLASS_NOTE && TList_[i].idx == idx) {
-			TListPtr_ = -1;
-			return;
-		}
+	TListPtr_++;
+	for (i = TListPtr_ - 1; i > 0; i--) {
+		TList_[i] = TList_[i-1];
+	}
+	TList_[0].evt_class = EVT_CLASS_REST;
+	TList_[0].start_time = start_time;
+	TList_[0].stop_time = stop_time;
+}
+
+void NMidiTimeScale::appendTidx(int idx) {
+	if (TListPtr_ >= TLIST_MAX) {
+		NResource::abort("appendTidx"); 
 	}
 	TList_[TListPtr_  ].evt_class = EVT_CLASS_NOTE;
 	TList_[TListPtr_++].idx = idx;
 }
 
-#define COST_FOR_REST 500
-#define HIGH_COSTS 100000
+void NMidiTimeScale::prependTidx(int idx) {
+	int i;
 
+	if (TListPtr_ >= TLIST_MAX) {
+		NResource::abort("prependTidx"); 
+	}
+	TListPtr_++;
+	for (i = TListPtr_ - 1; i > 0; i--) {
+		TList_[i] = TList_[i-1];
+	}
+	TList_[0].evt_class = EVT_CLASS_NOTE;
+	TList_[0].idx = idx;
+}
 
-int NMidiTimeScale::findTripletParts(int T0, int T1, int divis) {
-	unrolled_midi_events_str *ptr;
-	unsigned int i;
-	int idx0, idx1, idx2;
+bool NMidiTimeScale::is_nearby(unsigned int test_time, unsigned int target_time, int dist) {
+	int diff;
 
-#ifdef XXX
-	printf("(qu = %d(%d,%d), Suche Triplet von %d(%d,%d) - %d(%d,%d), divis = %d(%d,%d)\n", QUARTER_LENGTH / M3, QUARTER_LENGTH / MULTIPLICATOR, QUARTER_LENGTH, T0 / M3, T0 / MULTIPLICATOR, T0, T1 / M3, T1 / MULTIPLICATOR, T1, divis / M3, divis / MULTIPLICATOR, divis); fflush(stdout);
-#endif
-	TListPtr_ = 0;
-	if ((idx0 = search_for_event(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + divis)) >= 0) {
-		if ((idx1 = search_for_event(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			if ((idx2 = search_for_event(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-				addTidx(idx0);
-				addTidx(idx1);
-				addTidx(idx2);
-				return 0;
-			}
-			addTidx(idx0);
-			addTidx(idx1);
-			addTRest(T0 + 2 * divis, T1);
-			return ((divis / TRI16) * COST_FOR_REST);
-		}
-		if ((idx1 = search_for_event(TRIPLET_SNAP, T0 + divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTidx(idx0);
-			addTidx(idx1);
+	diff = test_time - target_time;
+	if (diff < 0) diff = -diff;
+	return diff <= dist / 2;
+}
+
+bool NMidiTimeScale::is_a_cuttable_right_note(int idx, unsigned int target_time) {
+	if (unrolled_midi_events_[idx].stop_time <= target_time) return false;
+	return true;
+}
+
+void NMidiTimeScale::append_cuttable_note(int idx, unsigned int split_time) {
+	if (TListPtr_ >= TLIST_MAX) {
+		NResource::abort("append_cuttable_note"); 
+	}
+	TList_[TListPtr_  ].evt_class = EVT_NOTE_AFTER_TRIPLET;
+	TList_[TListPtr_  ].split_time = split_time;
+	TList_[TListPtr_++].idx = idx;
+}
+
+void NMidiTimeScale::prepend_cuttable_note(int idx, unsigned int split_time) {
+	int i;
+
+	if (TListPtr_ >= TLIST_MAX) {
+		NResource::abort("prepend_cuttable_note"); 
+	}
+	TListPtr_++;
+	for (i = TListPtr_ - 1; i > 0; i--) {
+		TList_[i] = TList_[i-1];
+	}
+	TList_[0].evt_class = EVT_NOTE_BEFORE_TRIPLET;
+	TList_[0].split_time = split_time;
+	TList_[0].idx = idx;
+}
+
+int NMidiTimeScale::findBigRightTripletPartSloppy(int idx, unsigned int T1, int divis) {
+	int idx1;
+
+	if (is_nearby(unrolled_midi_events_[idx].stop_time, T1, unrolled_midi_events_[idx].stop_time -  unrolled_midi_events_[idx].U.norm_evt.triplet_start_time)) {
+		unrolled_midi_events_[idx].stop_time = T1;
+		appendTidx(idx);
+		return 0;
+	}
+
+	if (is_a_cuttable_right_note(idx, T1)) {
+		append_cuttable_note(idx, T1);
+		return COSTS_FOR_SPLIT;
+	}
+
+	if (is_nearby(unrolled_midi_events_[idx].U.norm_evt.triplet_stop_time, T1 - divis, unrolled_midi_events_[idx].U.norm_evt.triplet_stop_time - unrolled_midi_events_[idx].U.norm_evt.triplet_start_time)) {
+		unrolled_midi_events_[idx].U.norm_evt.triplet_stop_time = T1 - divis;
+		if ((idx1 = search_for_event_enlarge(TRIPLET_SNAP, T1 - divis, SYSTEM2_SNAP, T1)) >= 0) {
+			appendTidx(idx);
+			appendTidx(idx1);
 			return 0;
 		}
-		if ((idx2 = search_for_event(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTidx(idx0);
-			addTRest(T0 + divis, T0 + 2 * divis);
-			addTidx(idx2);
-			return ((divis / TRI16) * COST_FOR_REST);
-		}
-		addTidx(idx0);
-		addTRest(T0 + divis, T1);
-		return ((divis / TRI16) * COST_FOR_REST);
-	}
-	if ((idx2 = search_for_event(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-		if ((idx1 = search_for_event(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			addTidx(idx1);
-			addTidx(idx2);
-			return 0;
-		}
-		if ((idx1 = search_for_event(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			addTRest(T0, T0 + divis);
-			addTidx(idx1);
-			addTidx(idx2);
-			return ((divis / TRI16) * COST_FOR_REST);
-		}
-		addTRest(T0, T0 + 2 * divis);
-		addTidx(idx2);
-		return ((divis / TRI16) * COST_FOR_REST);
-	}
-	if ((idx2 = search_for_event(TRIPLET_SNAP, T0 + divis, SYSTEM2_SNAP, T1)) >= 0) {
-		addTRest(T0, T0 + divis);
-		addTidx(idx2);
-		return ((divis / TRI16) * COST_FOR_REST);
-	}
-	if ((idx1 = search_for_event(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-		if ((idx2 = search_for_event(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTRest(T0, T0 + divis);
-			addTidx(idx1);
-			addTRest(T0 + 2 * divis, T1);
-			return (2 * ((divis / TRI16) * COST_FOR_REST));
+		if ((idx1 = search_cuttable_note_right(T1, T1 - divis)) >= 0) {
+			appendTidx(idx);
+			append_cuttable_note(idx1, T1);
+			return COSTS_FOR_SPLIT;
 		}
+		appendTidx(idx);
+		appendTRest(T1 - divis, T1);
+		return COST_FOR_REST;
 	}
 	return HIGH_COSTS;
 }
 
-int NMidiTimeScale::findTripletPartsSloppy(int T0, int T1, int divis) {
-	unrolled_midi_events_str *ptr;
-	unsigned int i;
-	int idx0, idx1, idx2;
-	TListPtr_ = 0;
-	if ((idx0 = search_for_event_enlarge(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + divis)) >= 0) {
-		if ((idx1 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-				addTidx(idx0);
-				addTidx(idx1);
-				addTidx(idx2);
-				return 0;
-			}
-			addTidx(idx0);
-			addTidx(idx1);
-			addTRest(T0 + 2 * divis, T1);
-			return ((divis / TRI16) * COST_FOR_REST);
-		}
-		if ((idx1 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTidx(idx0);
-			addTidx(idx1);
+
+int NMidiTimeScale::findSmallLeftTripletPartSloppy(unsigned int T0, int divis) {
+	int idx1;
+
+	if ((idx1 = search_for_event_enlarge(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + divis)) >= 0) {
+		prependTidx(idx1);
+		return 0;
+	}
+	if ((idx1 = search_cuttable_note_left(T0, T0 + divis)) >= 0) {
+		prepend_cuttable_note(idx1, T0);
+		return COSTS_FOR_SPLIT;
+	}
+	prependTRest(T0, T0 + divis);
+	return COST_FOR_REST;
+}
+
+int NMidiTimeScale::findBigLeftTripletPartSloppy(unsigned int T0, int divis) {
+	int idx1, idx2;
+
+	if ((idx1 = search_for_event_enlarge(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
+		appendTidx(idx1);
+		return 0;
+	}
+
+	if ((idx1 = search_for_event_enlarge(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + divis)) >= 0) {
+		if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
+			appendTidx(idx1);
+			appendTidx(idx2);
 			return 0;
 		}
-		if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTidx(idx0);
-			addTRest(T0 + divis, T0 + 2 * divis);
-			addTidx(idx2);
-			return 0;
-		}
-		addTidx(idx0);
-		addTRest(T0 + divis, T1);
-		return ((divis / TRI16) * COST_FOR_REST);
-	}
-	if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-		if ((idx1 = search_for_event_enlarge(SYSTEM2_SNAP, T0, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			addTidx(idx1);
-			addTidx(idx2);
-			return 0;
-		}
-		if ((idx1 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-			addTRest(T0, T0 + divis);
-			addTidx(idx1);
-			addTidx(idx2);
-			return ((divis / TRI16) * COST_FOR_REST);
-		}
-		addTRest(T0, T0 + 2 * divis);
-		addTidx(idx2);
-		return ((divis / TRI16) * COST_FOR_REST);
-	}
-	if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, SYSTEM2_SNAP, T1)) >= 0) {
-		addTRest(T0, T0 + divis);
-		addTidx(idx2);
-		return ((divis / TRI16) * COST_FOR_REST);
+		appendTidx(idx1);
+		appendTRest(T0 + divis, T0 + 2 * divis);
+		return COST_FOR_REST;
+        }
+	if ((idx1 = search_cuttable_note_left(T0, T0 + 2 * divis)) >= 0) {
+		prepend_cuttable_note(idx1, T0 + 2 * divis);
+		return COSTS_FOR_SPLIT;
 	}
+
 	if ((idx1 = search_for_event_enlarge(TRIPLET_SNAP, T0 + divis, TRIPLET_SNAP, T0 + 2 * divis)) >= 0) {
-		if ((idx2 = search_for_event_enlarge(TRIPLET_SNAP, T0 + 2 * divis, SYSTEM2_SNAP, T1)) >= 0) {
-			addTRest(T0, T0 + divis);
-			addTidx(idx1);
-			addTRest(T0 + 2 * divis, T1);
-			return (2 * ((divis / TRI16) * COST_FOR_REST));
-		}
+                appendTRest(T0, T0 + divis);
+                appendTidx(idx1);
+		return COST_FOR_REST;
+        }
+	appendTRest(T0, T0 + 2 * divis);
+	return COST_FOR_REST;
+}
+
+int NMidiTimeScale::findSmallRightTripletPartSloppy(int idx, unsigned int T1, int divis) {
+	if (is_nearby(unrolled_midi_events_[idx].stop_time, T1, unrolled_midi_events_[idx].stop_time - unrolled_midi_events_[idx].U.norm_evt.triplet_start_time)) {
+		unrolled_midi_events_[idx].stop_time = T1;
+		appendTidx(idx);
+		return 0;
+	}
+	if (is_a_cuttable_right_note(idx, T1)) {
+		append_cuttable_note(idx, T1);
+		return COSTS_FOR_SPLIT;
 	}
 	return HIGH_COSTS;
 }
 
-
-void NMidiTimeScale::searchForTriplet(int tripletLength, int snapedTripletPosition, int typeFound) {
-	int i, j, k, n, divis, costs, minT0, minT1, T0, T1, mincosts;
+void NMidiTimeScale::searchForTriplet(int idx, int tripletLength, int snapedTripletPosition, int typeFound) {
+	int i, j, k, n, divis, costs, minT0, minT1, mincosts;
 	struct unrolled_midi_events_str pseudo_note, *ptr2, *ptr3;
+	unsigned int T0, T1;
 	int num_pitches;
 	int Tl, tyF;
 
 #ifdef XXX
-	printf("1/%d Zerlegung bei (%d/3), n = %d\n", 8 / tripletLength, typeFound + 1, snapedTripletPosition); fflush(stdout);
+	printf("int idx = %d, 1/%d Zerlegung bei (%d/3), n = %d\n", idx, 8 / tripletLength, typeFound + 1, snapedTripletPosition); fflush(stdout);
 #endif
 
 
@@ -864,19 +979,23 @@ void NMidiTimeScale::searchForTriplet(int tripletLength, int snapedTripletPositi
 	mincosts = (1 << 30);
 	snapedTripletPosition *= TRI16;
 	while (Tl >= 1) {
+		TListPtr_ = 0;
 		divis = Tl * TRI16;
 		if (tyF) {
 			T0 = snapedTripletPosition - 2 * divis;
 			T1 = snapedTripletPosition + divis;
 			if (T0 & 0x80000000) continue; // < 0
+			costs  = findBigLeftTripletPartSloppy(T0, divis);
+			costs += findSmallRightTripletPartSloppy(idx, T1, divis);
 		}
 		else {
 			T0 = snapedTripletPosition - divis;
 			T1 = snapedTripletPosition + 2 * divis;
 			if (T0 & 0x80000000) continue; // < 0
+			costs  = findBigRightTripletPartSloppy(idx, T1, divis);
+			costs += findSmallLeftTripletPartSloppy(T0, divis);
 		}
-		costs = findTripletParts(T0, T1, divis);
-		if (costs < mincosts && TListPtr_ > 0) {
+		if (costs < mincosts  && costs < HIGH_COSTS && TListPtr_ > 0) {
 			mincosts = costs;
 			minT0 = T0;
 			minT1 = T1;
@@ -886,37 +1005,12 @@ void NMidiTimeScale::searchForTriplet(int tripletLength, int snapedTripletPositi
 		Tl >>= 1;
 		tyF = 1 - tyF;
 	}
-	if (MinTListPtr_ < 0) {
-		Tl = tripletLength;
-		mincosts = (1 << 30);
-		tyF = typeFound;
-		while (Tl >= 1) {
-			divis = Tl * TRI16;
-			if (tyF) {
-				T0 = snapedTripletPosition - 2 * divis;
-				T1 = snapedTripletPosition + divis;
-				if (T0 & 0x80000000) continue; // < 0
-			}
-			else {
-				T0 = snapedTripletPosition - divis;
-				T1 = snapedTripletPosition + 2 * divis;
-				if (T0 & 0x80000000) continue; // < 0
-			}
-			costs = findTripletPartsSloppy(T0, T1, divis);
-			if (costs < mincosts && TListPtr_ > 0) {
-				mincosts = costs;
-				minT0 = T0;
-				minT1 = T1;
-				MinTListPtr_ = TListPtr_;
-				memcpy (MinTList_, TList_, TListPtr_ * sizeof(struct tripletMemberStr));
-			}
-			Tl >>= 1;
-			tyF = 1 - tyF;
-		}
-	}
 
 	if (MinTListPtr_ >= 0) {
-		printf("Triplet\n"); fflush(stdout);
+#ifdef XXX
+		printf("Triplet anzahl  = %d minT0 = %d(%d), minT1 = %d(%d)\n", MinTListPtr_, minT0 / MULTIPLICATOR, minT0,
+		minT1 / MULTIPLICATOR, minT1);  fflush(stdout);}
+#endif
 		num_pitches = 0;
 		pseudo_note.eventType = EVT_PSEUDO_TRIPLET_NOTE | EVT_CLASS_NOTE;
 		pseudo_note.voice_nr = -1;
@@ -926,17 +1020,25 @@ void NMidiTimeScale::searchForTriplet(int tripletLength, int snapedTripletPositi
 		pseudo_note.stop_time = minT1;
 		pseudo_note.U.pseudo_evt.num_triplet_members = MinTListPtr_;
 		for (i = 0; i < MinTListPtr_; i++) {
-			if (MinTList_[i].evt_class & EVT_CLASS_NOTE) {
+			if (MinTList_[i].evt_class & (EVT_CLASS_NOTE | EVT_NOTE_AFTER_TRIPLET | EVT_NOTE_BEFORE_TRIPLET)) {
 				ptr2 = &(unrolled_midi_events_[MinTList_[i].idx]);
 				if ((ptr2->eventType & EVT_NORMAL_EVENT) == 0) {
 					NResource::abort("NMidiTimeScale::checkForTripletMembers", 1);
 				}
 				if (i == 0) {
 					ptr2->eventType = EVT_FIRST_IN_TRIPLET | EVT_CLASS_NOTE;
+					if (MinTList_[i].evt_class & EVT_NOTE_BEFORE_TRIPLET) {
+						ptr2->eventType |= EVT_NOTE_BEFORE_TRIPLET;
+						ptr2->split_time = MinTList_[i].split_time;
+					}
 					ptr2->U.norm_evt.triplet_border = ptr2->U.norm_evt.used_part_time = minT0;
 				}
 				else if (i == MinTListPtr_ - 1) {
 					ptr2->eventType = EVT_LAST_IN_TRIPLET | EVT_CLASS_NOTE;
+					if (MinTList_[i].evt_class & EVT_NOTE_AFTER_TRIPLET) {
+						ptr2->eventType |= EVT_NOTE_AFTER_TRIPLET;
+						ptr2->split_time = MinTList_[i].split_time;
+					}
 					ptr2->U.norm_evt.triplet_border = ptr2->U.norm_evt.used_part_time = minT1;
 				}
 				else {
@@ -980,7 +1082,6 @@ void NMidiTimeScale::searchForTriplet(int tripletLength, int snapedTripletPositi
 }
 
 
-
 void NMidiTimeScale::findTriplets() {
 	unrolled_midi_events_str *ptr;
 	unsigned int i, snapedTipletPosition;
@@ -1009,25 +1110,26 @@ void NMidiTimeScale::findTriplets() {
 	fflush(stdout);
 #endif
 
-	for (k = 1; k <= 8; k <<= 1) {
+	for (k = mink_; k <= 8; k <<= 1) {
 		for (i = 0; i < array_len_; i++) {
 			ptr = &(unrolled_midi_events_[i]);
 			if ((ptr->eventType & EVT_NORMAL_EVENT) == 0) continue;
 			if (ptr->U.norm_evt.sta3diff >= ptr->U.norm_evt.sta2diff) continue;
+			//snapedTipletPosition = (ptr->U.norm_evt.triplet_start_time - lastTimeSigTime(ptr->U.norm_evt.triplet_start_time)) / TRI16;
 			snapedTipletPosition = ptr->U.norm_evt.triplet_start_time / TRI16;
 			if ((snapedTipletPosition % 3) && !(snapedTipletPosition % k) && ((snapedTipletPosition % (k << 1)) || (k == 8))) {
 				typeFound = ((snapedTipletPosition - k) % 3) ? 1 : 0;
-				searchForTriplet(k, snapedTipletPosition, typeFound);
+				searchForTriplet(i, k, snapedTipletPosition, typeFound);
 			}
 		}
 	}
 }
 
-#endif
-
+#define SNAP1(x, D, L) (((x - L + D / 2) / D) * D + L) 
 
 void NMidiTimeScale::insertMidiEvent(TSE3::MidiEvent *midiEvent,  unsigned int min, unsigned int max) {
 	struct unrolled_midi_events_str unr_evt;
+	unsigned int lastTimesig;
 	int len;
 	unsigned int val2, val3, snap2, snap3;
 	switch(midiEvent->data.status) {
@@ -1037,14 +1139,42 @@ void NMidiTimeScale::insertMidiEvent(TSE3::MidiEvent *midiEvent,  unsigned int m
 			unr_evt.U.norm_evt.pitches[0] = midiEvent->data.data1;
 			unr_evt.start_time = TSE3TIME2MYMIDITIME(midiEvent->time.pulses);
 			unr_evt.stop_time = TSE3TIME2MYMIDITIME(midiEvent->offTime.pulses);
+
+			if (unr_evt.start_time & (1 << 31)) { // negative, appens during import from keyboard
+				if (-unr_evt.start_time > 128) {
+					unr_evt.start_time = 0;
+				}
+				else {
+					return;
+				}
+			}
+			if (unr_evt.stop_time & (1 << 31)) { // negative, appens during import from keyboard
+				return;
+			}
 			unr_evt.U.norm_evt.volume = midiEvent->data.data2;
 			unr_evt.U.norm_evt.num_pitches = 1;
 			unr_evt.voice_nr = -1;
 			len = unr_evt.stop_time - unr_evt.start_time;
-			unr_evt.U.norm_evt.rawlen = snap2 = determine_snap(len);
-			snap3 = snap2 * 2 / 3;
+#ifdef XXX
+#define M(x) ((x) / MULTIPLICATOR)
+			printf("On = %d, Off = %d, len = %d, star = %d, stop = %d ->",
+			midiEvent->time.pulses, midiEvent->offTime.pulses, M(len), M(unr_evt.start_time), M(unr_evt.stop_time));
+			fflush(stdout);
+#endif
+			if (noteSnap_ < 0) {
+				snap2 = determine_snap(len);
+			}
+			else {
+				snap2 = noteSnap_;
+			}
+			snap3 = snap2 * 4 / 3;
+			lastTimesig = lastTimeSigTime(unr_evt.start_time + snap2);
+			val2 = SNAP1(unr_evt.start_time, snap2, lastTimesig);
+			val3 = SNAP1(unr_evt.start_time, snap3, lastTimesig);
+			/*
 			val2 = ((unr_evt.start_time + snap2 / 2) / snap2) * snap2;
 			val3 = ((unr_evt.start_time + snap3 / 2) / snap3) * snap3;
+			*/
 			unr_evt.U.norm_evt.sta2diff = val2 - unr_evt.start_time;
 			if (unr_evt.U.norm_evt.sta2diff < 0) unr_evt.U.norm_evt.sta2diff = -unr_evt.U.norm_evt.sta2diff;
 			unr_evt.U.norm_evt.sta3diff = val3 - unr_evt.start_time;
@@ -1060,14 +1190,27 @@ void NMidiTimeScale::insertMidiEvent(TSE3::MidiEvent *midiEvent,  unsigned int m
 #endif
 			unr_evt.start_time = val2;
 			unr_evt.U.norm_evt.triplet_start_time = val3;
+			//lastTimesig = lastTimeSigTime(unr_evt.stop_time);
+			//snap2 >>= 1;
+			//snap3 >>= 1;
+			val2 = SNAP1(unr_evt.stop_time, snap2, lastTimesig);
+			val3 = SNAP1(unr_evt.stop_time, snap3, lastTimesig);
+			/*
 			val2 = ((unr_evt.stop_time + snap2 / 2) / snap2) * snap2;
 			val3 = ((unr_evt.stop_time + snap3 / 2) / snap3) * snap3;
-			unr_evt.U.norm_evt.sto2diff = val2 - unr_evt.stop_time;
-			if (unr_evt.U.norm_evt.sto2diff < 0) unr_evt.U.norm_evt.sto2diff = -unr_evt.U.norm_evt.sto2diff;
-			unr_evt.U.norm_evt.sto3diff = val3 - unr_evt.stop_time;
-			if (unr_evt.U.norm_evt.sto3diff < 0) unr_evt.U.norm_evt.sto3diff = -unr_evt.U.norm_evt.sto3diff;
+			*/
+			if (val2 == unr_evt.start_time && noteSnap_ > 0) {
+				if (unr_evt.stop_time - unr_evt.start_time > noteSnap_ / 4) {
+					val2 = unr_evt.start_time + noteSnap_;
+				}
+			}
 			unr_evt.stop_time = val2;
 			unr_evt.U.norm_evt.triplet_stop_time = val3;
+#ifdef XXX
+			printf("len = %d, star = %d, stop = %d\n",
+			M(unr_evt.stop_time-unr_evt.start_time), M(unr_evt.start_time), M(unr_evt.stop_time));
+			fflush(stdout);
+#endif
 			insertEvent(&unr_evt);
 			break;
 		case TSE3::MidiCommand_ProgramChange:
@@ -1084,7 +1227,7 @@ void NMidiTimeScale::insertMidiEvent(TSE3::MidiEvent *midiEvent,  unsigned int m
 }
 
 
-void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props_str *staff_props, NClef *clef, int stem_policy,
+void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props_str *staff_props, NClef *clef,
 			NVoice *voice, bool first, bool drum_channel, int volmindist, bool computeAverageVolume, unsigned int actualVolume, double averageVolume, double dynamic) {
 	unsigned int i, j, l;
 	int len = 0, len2, len3, newlen;
@@ -1100,6 +1243,7 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 	bool triplet = false;
 	bool addRest;
 	bool stacatto;
+	bool potential_after_triplet, before_triplet = false;
 	int line, offs;
 	NChord *chord = 0, *chord2;
 	NRest *rest;
@@ -1166,21 +1310,17 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 #endif
 			}
 			else {
+				voiceTime += len;
 				len = 0;
 			}
 		}
 		stop = ((ptr->eventType & EVT_CLASS_REST) || (ptr->eventType & (EVT_NORMAL_EVENT |  EVT_LAST_IN_TRIPLET))) ?  ptr->stop_time : ptr->U.norm_evt.triplet_stop_time;
 		status = 0;
 		len = stop - start;
-
-#ifdef WITH_TRIPLET_RECOGNITION
 		triplet = (ptr->eventType & (EVT_PART_OF_TRIPLET | EVT_LAST_IN_TRIPLET | EVT_FIRST_IN_TRIPLET));
-#else
-		triplet = false;
-#endif
 		stacatto = false;
 		following_rest_len = 0;
-		if (!triplet && !inTriplet && len <= QUARTER_LENGTH) {
+		if (!triplet && !inTriplet && len < QUARTER_LENGTH) {
 			for (next_found = false, nptr = ptr + 1, j = i + 1; j < l; j++,  nptr++) {
 				if (nptr->eventType & EVT_PROGRAM_CHANGE) continue;
 				if (nptr->voice_nr != nr) continue;
@@ -1194,30 +1334,35 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 				newlen = following_rest_len + len;
 				len2 = quantNote(newlen, &dotcount, DOUBLE_WHOLE_LENGTH);
 				len3 = dotcount ? 3 * len2 / 2 : len2;
-				if (len3 <= newlen && len3 < 5 * len / 4) {
+				if (len3 <= newlen && len3 < 3 * len) {
 					stacatto = (len3 >= 2*len);
 					stop = start + len;
 					len = len3;	
 				}
 			}
 		}
-
 		chord = 0;
+		potential_after_triplet = false;
+		if (ptr->eventType & EVT_NOTE_BEFORE_TRIPLET) {
+			len = ptr->split_time - ptr->start_time;
+			before_triplet = true;
+		}
+		if (ptr->eventType & EVT_NOTE_AFTER_TRIPLET) {
+			potential_after_triplet = true;
+			len = ptr->split_time - ptr->U.norm_evt.triplet_start_time;
+		}
 		while (len >= NOTE128_LENGTH) {
-#ifdef WITH_TRIPLET_RECOGNITION
-			if (triplet) {
+			if (triplet && !before_triplet) {
 				len2 = quantTriplet(len, ptr, &addRest, WHOLE_LENGTH);
 				dotcount = 0;
 			}
 			else {
-#endif
 				addRest = false;
 				len2 = quantNote(len, &dotcount, DOUBLE_WHOLE_LENGTH);
-#ifdef WITH_TRIPLET_RECOGNITION
+
 			}
-#endif
 			if (len2 >= NOTE128_LENGTH) {
-				if (triplet) {
+				if (triplet && !before_triplet) {
 					len3 = 2 * len2 / 3;
 				}
 				else if (dotcount) {
@@ -1264,6 +1409,7 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 					}
 					if (chord) {
 						chord2 = chord->clone();
+						chord2->unsetTuplet();
 						chord2->tieWith(chord);
 						chord2->changeLength(len2);
 						if (dotcount) {
@@ -1286,17 +1432,17 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 						if (stacatto) {
 							status |= STAT_STACC;
 						}
-						chord = new NChord(main_props, staff_props, line, offs, len2, stem_policy, status | body);
+						chord = new NChord(main_props, staff_props, line, offs, len2, ptr->stem_policy, status | body);
 						for (j = 1; j < ptr->U.norm_evt.num_pitches; j++) {
 							clef->midi2Line(ptr->U.norm_evt.pitches[j], &line, &offs);
 							body = 0;
 							if (drum_channel) {
 								body = (STAT_BODY_CROSS << ((line + LINE_OVERFLOW) % 5));
 							}
-							chord->insertNewNote(line, offs, stem_policy, status | body);
+							chord->insertNewNote(line, offs, ptr->stem_policy, status | body);
 						}
 						voice->appendElem(chord);
-						if (triplet) {
+						if (triplet && !before_triplet)  {
 							if (!inTriplet) {
 								tupletList = new QList<NMusElement>();
 								inTriplet = true;
@@ -1304,9 +1450,30 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 							tupletList->append(chord);
 						}
 					}
+					if (len < NOTE128_LENGTH) {
+						if (before_triplet) {
+							before_triplet = false;
+							len = stop - ptr->split_time;
+							tupletList = new QList<NMusElement>();
+							inTriplet = true;
+						}
+						if (potential_after_triplet) {
+							potential_after_triplet = false;
+							len = stop - ptr->split_time;
+							if (tupletList->count() < 2) {
+								tupletList->clear();
+							}
+							else {
+								NMusElement::computeTuplet(tupletList, 3, 2);
+							}
+							tupletList = 0;
+							triplet = inTriplet = false;
+						}
+					}
 				}
 			}
 			else {
+				voiceTime += len;
 				len = 0;
 			}
 		}
@@ -1321,55 +1488,7 @@ void NMidiTimeScale::createVoice(int nr, main_props_str *main_props, staff_props
 			inTriplet = false;
 		}
 	}
-}
-
-void NMidiTimeScale::sort_voices(int *voicemap) {
-#define PITCH_FAC 100.0
-#define COUNT_FAC 20.0
-	unsigned int i, l, k;
-	int idx, j, n, tmp;
-	unrolled_midi_events_str *ptr;
-	double max_rating, sum;
-	double rating[MAX_VOICES];
-	int num_pitches;
-	int max_ave_idx;
-
-	l = array_len_;
-	for (n = 0; n < max_voices_; n++) {
-		num_pitches = 1;
-		sum = 0.0;
-		for (i = 0, ptr = &(unrolled_midi_events_[0]); i < l; i++, ptr++) {
-			if ((ptr->eventType & (EVT_NORMAL_EVENT | EVT_PART_OF_TRIPLET | EVT_LAST_IN_TRIPLET)) == 0) continue;
-			if (ptr->eventType & EVT_CLASS_REST) continue;
-			if (ptr->voice_nr != n) continue;
-			for (k = 0; k < ptr->U.norm_evt.num_pitches; k++) {
-				sum += ptr->U.norm_evt.pitches[k];
-				num_pitches++;
-			}
-		}
-		rating[n] = PITCH_FAC * sum / (double) num_pitches + COUNT_FAC * num_pitches; 
-	}
-
-	max_rating = 0.0;
-	for (i = 0; i < MAX_VOICES; i++) {
-		voicemap[i] = i;
-	}
-
-	for (idx = 0; idx < max_voices_ - 1; idx++) {
-		max_rating = rating[idx];
-		max_ave_idx = idx;
-		for (j = idx + 1; j < max_voices_; j++) {
-			if (rating[j] > max_rating) {
-				max_rating = rating[j];
-				max_ave_idx = j;
-			}
-		}
-		if (max_ave_idx != idx) {
-			tmp = voicemap[idx];
-			voicemap[idx] = voicemap[max_ave_idx];
-			voicemap[max_ave_idx] = tmp;
-		}
-	}
+	
 }
 
 
@@ -1379,7 +1498,6 @@ void NMidiTimeScale::createStaff(NStaff *staff, bool drum_channel, int volmindis
 	NClef *clef;
 	main_props_str *main_props;
 	staff_props_str *staff_props;
-	int voice_map[MAX_VOICES];
 
 	clef = &(staff->actualClef_);
 	main_props = staff->getVoiceNr(0)->getMainPropsAddr();
@@ -1393,17 +1511,17 @@ void NMidiTimeScale::createStaff(NStaff *staff, bool drum_channel, int volmindis
 	printf("%d voices\n", voice_count());
 	fflush(stdout);
 #endif
-#ifdef WITH_TRIPLET_RECOGNITION
-	findTriplets();
-#endif
+	if (smallestTupletNote_ != -1) findTriplets();
 	findVoices();
-	sort_voices(voice_map);
+	while (staff->voicelist_.at(1)) {
+		staff->voicelist_.remove();
+	}
 	if (max_voices_ > 1) {
 		staff->addVoices(max_voices_ - 1);
 	}
 	for (i = 0; i < max_voices_; i++) {
 		voice = staff->getVoiceNr(i);
-		createVoice(voice_map[i], main_props, staff_props, clef, (max_voices_ == 1) ? STEM_POL_INDIVIDUAL : ((i >= max_voices_ / 2) ? STEM_POL_DOWN : STEM_POL_UP), voice, i == 0, drum_channel, volmindist, computeAverageVolume, actualVolume, averageVolume, dynamic);
+		createVoice(i, main_props, staff_props, clef, voice, i == 0, drum_channel, volmindist, computeAverageVolume, actualVolume, averageVolume, dynamic);
 	}
 }
 #endif
