@@ -258,7 +258,6 @@ void NMusicXMLExport::debugDump(QList<NStaff> *stafflist, NMainFrameWidget *main
 	NStaff *staff_elem;
 //	int staffcount;
 	int i;
-//	extern char *instrTab[];
 	for (i = 0, staff_elem = stafflist->first(); staff_elem; i++, staff_elem = stafflist->next()) {
 		out_ << "*** Staff " << i
 			<< " staffName=" << staff_elem->staffName_
@@ -1640,7 +1639,6 @@ void NMusicXMLExport::outputStaffAndVoiceDescription(QList<NStaff> *stafflist) {
 	NStaff *staff_elem;
 	int staffcount;
 	int i;
-	extern char *instrTab[];
 
 	staffcount = stafflist->count();
 	out_ << endl;
@@ -1653,7 +1651,7 @@ void NMusicXMLExport::outputStaffAndVoiceDescription(QList<NStaff> *stafflist) {
 			out_ << "\t\t\t<part-name>" << "Staff " << i+1 << "</part-name>\n";
 		}
 		out_ << "\t\t\t<score-instrument id=\"P" << i+1 << "-I" << i+1 << "\">\n";
-		out_ << "\t\t\t\t<instrument-name>" << i18n( instrTab[staff_elem->getVoice()] )
+		out_ << "\t\t\t\t<instrument-name>" << i18n( NResource::instrTab[staff_elem->getVoice()] )
 			<< "</instrument-name>\n";
 		out_ << "\t\t\t</score-instrument>\n";
 		out_ << "\t\t\t<midi-instrument id=\"P" << i+1 << "-I" << i+1 << "\">\n";
