@@ -59,6 +59,7 @@
 #include "chord.h"
 #include "chorddiagram.h"
 #include "outputbox.h"
+#include "text.h"
 #include "musixhint.h"
 #include "multistaffinfo.h"
 #include "layout.h"
@@ -73,6 +74,7 @@ NMusiXTeX::NMusiXTeX() {
 	non_shifted_notes_.setAutoDelete(false);
 	pending_key_changes_.setAutoDelete(true);
 	pending_time_changes_.setAutoDelete(true);
+	pending_texts_.setAutoDelete(true);
 }
 
 
@@ -172,6 +174,7 @@ void NMusiXTeX::doExport() {
 		dysymDescr_[i].lastDynSym = 0;
 		dysymDescr_[i].dynEndPos = 0;
 	}
+	pending_texts_.clear();
 	if (fileName.isNull())
 		return;
 	out_.open(fileName);
@@ -767,8 +770,8 @@ void NMusiXTeX::generate(int staff_nr, int real_staff_nr, const char *extraDelim
 	lastSignWasBar_ = false;
 	lastSignWasRepeatClose_ = false;
 	switch(elem->getType()) {
-		case T_CHORD: 
-			     if (extraDelimiter) out_ << extraDelimiter;
+		case T_CHORD: if (extraDelimiter) out_ << extraDelimiter;
+			     writeStaffTexts(real_staff_nr);
 			     chord = (NChord *) elem;
 			     chord->initialize_acc_pos_computation();
 			     if ((chord->status_ & STAT_GRACE) && noteSizeIsNormal_) {
@@ -1108,7 +1111,8 @@ void NMusiXTeX::generate(int staff_nr, int real_staff_nr, const char *extraDelim
 				if (i % 2) out_ << "\\hsk";
 			     }
 			     break;
-		case T_REST: if (elem->status_ & STAT_HIDDEN) break;
+		case T_REST: writeStaffTexts(real_staff_nr);
+			     if (elem->status_ & STAT_HIDDEN) break;
 			     if (extraDelimiter) out_ << extraDelimiter;
 			     if (staff_nr == 0 && newTempo_ >= 0) {
 					out_ << "\\Uptext{\\metron{\\nq}{" << newTempo_ << "}}";
@@ -1285,6 +1289,29 @@ void NMusiXTeX::generate(int staff_nr, int real_staff_nr, const char *extraDelim
 			     		break;
 			     }
 			     break;
+		case T_TEXT: pending_texts_.append(new textDescr((NText *) elem, real_staff_nr));
+			     break;
+	}
+}
+
+void NMusiXTeX::writeStaffTexts(int real_staff_nr) {
+	int pos;
+	textDescr *text_tescr;
+	text_tescr = pending_texts_.first();
+	while (text_tescr) {
+		if (text_tescr->staff_nr != real_staff_nr)  {
+			text_tescr = pending_texts_.next();
+			continue;
+		}
+		if (text_tescr->textElem->getSubType() == TEXT_UPTEXT) {
+			pos = 11;
+		}
+		else {
+			pos = -4;
+		}
+		out_ << "\\zcharnote{" << pos << "}{\\textbf{" << text_tescr->textElem->getText() << "}}";
+		pending_texts_.remove();
+		text_tescr = pending_texts_.first();
 	}
 }
 
