@@ -38,6 +38,7 @@
 #include "rest.h"
 #include "chord.h"
 #include "staff.h"
+#include "text.h"
 #include "filehandler.h"
 #include "uiconnect.h"
 #include "scaleedit_impl.h"
@@ -497,6 +498,8 @@ bool NPmxExport::writeTrack(NVoice *voice,  int staff_nr, int voice_nr, int voic
 	bool inGraceGroup = false;
 	bool nested, toomany;
 	QString graceString;
+	NText *textElem;
+	char pos;
 
 	elem = voice->getCurrentPosition();
 	if (elem == 0) return false;
@@ -905,6 +908,16 @@ bool NPmxExport::writeTrack(NVoice *voice,  int staff_nr, int voice_nr, int voic
 					handleSpecialChar(posOfpendingKeySig_, &specialChar, voice_count, false);
 				}
 				break;
+			case T_TEXT:
+				textElem = (NText *) elem;
+				if (textElem->getSubType() == TEXT_UPTEXT) {
+					pos = actual_staff->actualClef_.line2TexTab_[11+LINE_OVERFLOW];
+				}
+				else {
+					pos = actual_staff->actualClef_.line2TexTab_[-3+LINE_OVERFLOW];
+				}
+				*pmxout_  << "\\zcharnote{" << pos << "}{\\bf " << textElem->getText() << "}\\relax\\ ";
+				break;
 		}
 		elem = voice->getNextPosition();
 	}
