@@ -70,19 +70,26 @@ using namespace std;
 
 #define BEAM_LINE_WIDTH 2
 
-char *volume[] =
+char *NResource::volume[] =
 {I18N_NOOP("ppp"), I18N_NOOP("pp"), I18N_NOOP("p"),  I18N_NOOP("mp"),
  I18N_NOOP("mf"),  I18N_NOOP("f"),  I18N_NOOP("ff"), I18N_NOOP("fff")
 };
 
-char *noteVal[] =
+char *NResource::noteVal[] =
 {I18N_NOOP("full"),         I18N_NOOP("dotted half"), I18N_NOOP("half"),
  I18N_NOOP("dotted quarter"), I18N_NOOP("quarter"),   I18N_NOOP("dotted 8th"),
  I18N_NOOP("8th"),          I18N_NOOP("dotted 16th"), I18N_NOOP("16th"),
  I18N_NOOP("dotted 32th"),  I18N_NOOP("32th"),        I18N_NOOP("64th")
 };
 
-char *instrTab[128] =
+char *NResource::tripletVal[] =
+{I18N_NOOP("half"),
+ I18N_NOOP("quarter"),
+ I18N_NOOP("8th"),
+ I18N_NOOP("16th")
+};
+
+char *NResource::instrTab[128] =
 {I18N_NOOP("Piano 1"),          I18N_NOOP("Piano 2"),
  I18N_NOOP("Piano 3"),          I18N_NOOP("Honky-tonk"),
  I18N_NOOP("E.Piano 1"),        I18N_NOOP("E.Piano 2"),
@@ -273,7 +280,7 @@ QString  NResource::fine_ = "Fine";
 QString  NResource::ritardando_ = "ritard.";
 QString  NResource::accelerando_ = "accel.";
 
-QFont    *NResource::text_;
+QFont    *NResource::textFont_;
 
 QPixmap  *NResource::crossPixmap_;
 QPixmap  *NResource::crossRedPixmap_;
@@ -960,8 +967,8 @@ NResource::NResource() {
 	flagDownPixmapHeight_ = flagDownPixmap_->height();
 	flagPixmapWidth_ = flagPixmap_->width();
 	flagPixmapWidth_ = tinyFlagPixmap_->width();
-//	text_ = new  QFont ("Times" , 36, QFont::Normal, false, QFont::ISO_8859_1 );  
-	text_ = new  QFont ("Times" , 36, QFont::Bold, false );  
+//	textFont_ = new  QFont ("Times" , 36, QFont::Normal, false, QFont::ISO_8859_1 );  
+	textFont_ = new  QFont ("Times" , 36, QFont::Bold, false );  
 
 	// check current LilyPond version to decide about the export format.
 	lilytest lt;
