@@ -67,9 +67,6 @@
 #endif
 #include "musicxmlexport.h"
 
-extern char *instrTab[];
-extern char *noteVal[];
-extern char *volume[];
 
 /*-------------------------------- line sel ----------------------------------*/
 
@@ -367,7 +364,7 @@ void exportFrm::setSaveHeight(int height) {pHeight->setValue(height);}
 staffPropFrm::staffPropFrm(QWidget *parent)  : staffPropForm(parent, 0, true)
 {
 	for (int i = 0; i < 128; ++i)
-		staffVoice->insertItem(i18n("%1. %2").arg(i).arg(i18n(instrTab[i])));
+		staffVoice->insertItem(i18n("%1. %2").arg(i).arg(i18n(NResource::instrTab[i])));
 	for (int i = 0; i < 16; ++i)
 		staffChannel->insertItem(i18n("Channel %1").arg(i + 1 ));
 
@@ -532,16 +529,24 @@ int scaleFrm::boot() {
 
 filterFrm::filterFrm( NMainFrameWidget *parent, bool modal ) : filterForm( parent, 0, modal ) {
 	averageSlot();
+	this->filSnapDist->insertItem(i18n("Auto"));
+	this->filSmTripletNote->insertItem(i18n("Auto"));
+	this->filSmTripletNote->insertItem(i18n("None"));
 	for (int i = 0; i < 12; ++i) {
+		this->filSnapDist->insertItem(i18n(NResource::noteVal[i]));
+	}
+	for (int i = 0; i < 4; ++i) {
+		this->filSmTripletNote->insertItem(i18n(NResource::tripletVal[i]));
+	}
 	this->filImpRe->setFocus();
 	this->filVal1->setAll(0, 127, 0);
 	this->filVal2->setAll(0, 127, 127);
 	this->filVelSca->setAll(0, 200, 100);
 	this->filVolDist->setAll(0, 127, 10);
-}
 
     mainWidget_ = parent;
     connect( filTSE, SIGNAL( clicked() ), parent, SLOT( TSE3toScore() ) );
+    connect( filTSEPart, SIGNAL( clicked() ), parent, SLOT( TSE3ParttoScore() ) );
     
     }
 
@@ -610,8 +615,8 @@ void metronomFrm::abortSlot() {
 /*---------------------------------------- smallestRestForm -------------------------------------*/
 
 smallestRestFrm::smallestRestFrm( NMainFrameWidget *parent ) : clRestForm( parent, 0, true ) {
-	for (unsigned int i = 0; noteVal[i]; ++i)
-	this->sel->insertItem(i18n(noteVal[i]));
+	for (unsigned int i = 0; NResource::noteVal[i]; ++i)
+	this->sel->insertItem(i18n(NResource::noteVal[i]));
 	this->btOk->setFocus();
 }
 
@@ -669,7 +674,7 @@ int smallestRestFrm::item2length() {
 
 volumeFrm::volumeFrm( QWidget *parent ) : volumeForm( parent, 0, true ) {
 	for (int i = 0; i < 8; ++i)
-	this->sel->insertItem(i18n(volume[i]));
+	this->sel->insertItem(i18n(NResource::volume[i]));
 	this->btOk->setFocus();
 	this->scal_ed->setAll(0, 127, 100);
 }
@@ -712,12 +717,12 @@ bool listFrm::boot( int val, short int type, const QString & caption, const QStr
     switch( type ) {
 	case LIST_VOICE: {
 	    for( int i = 0; i < 128; ++i )
-		this->choice->insertItem(i18n("%1. %2").arg(i).arg(i18n(instrTab[i])));
+		this->choice->insertItem(i18n("%1. %2").arg(i).arg(i18n(NResource::instrTab[i])));
 	    } break;
 	case LIST_MOVE_STAFF: {
 	    NStaff *stf = staff->first();
 	    for( int i = 1; stf; stf = staff->next(), i++ )
-		this->choice->insertItem(i18n("Staff %1, %2, %3").arg(i).arg(i18n(instrTab[stf->getVoice()])).
+		this->choice->insertItem(i18n("Staff %1, %2, %3").arg(i).arg(i18n(NResource::instrTab[stf->getVoice()])).
 								arg( stf->staffName_ == "" || stf->staffName_ == QString::null  ?
 						        i18n("[unnamed]") : QString(stf->staffName_)));
 	    } break;
@@ -955,7 +960,7 @@ void staffFrm::boot( QList<NStaff> *stafflist, char unsigned id, int amount ) {
 		for(NStaff *staff_elem = stafflist->first(); staff_elem; staff_elem = stafflist->next(), ++i ) {
 			items_[i] = new QCheckListItem
 				((QListView *) this->elem, i18n("Staff %1%2").arg(i < 9 ? " " : "").arg(i + 1), QCheckListItem::CheckBox);
-			items_[i]->setText(1, i18n(instrTab[staff_elem->getVoice()]));
+			items_[i]->setText(1, i18n(NResource::instrTab[staff_elem->getVoice()]));
 			items_[i]->setText( 2, ( staff_elem->staffName_ == "" || staff_elem->staffName_ == QString::null  ?
 			i18n("[unnamed]") : QString(staff_elem->staffName_)));
 		}
