@@ -42,6 +42,7 @@ class NClef;
 class NKeySig;
 class NChordDiagram;
 class NTempoTrack;
+class NText;
 
 class NPositStr {
 	public:
@@ -113,6 +114,7 @@ class NVoice {
 /*------------------------ getting voice proerties ---------------------------------------*/
 		main_props_str *getMainPropsAddr() {return main_props_;}
 		NStaff *getStaff() {return theStaff_;}
+		void printAll();
 		void setActualStemDir(int stemDir) {main_props_->actualStemDir = stemDir;}
 		void getTempoSigs(NTempoTrack *ttrack, int startTime);
 		bool isFirstVoice() {return firstVoice_;}
@@ -249,6 +251,7 @@ class NVoice {
 		void resetSpecialElement();
 		void syncSpecialElement(int xpos);
 		NMusElement *findBarInStaff(int start_time, int stop_time);
+		NMusElement *findBarInStaffTillXpos(int start_time, int endXpos);
 		void mark();
 		void gotoMarkedPosition();
 		NMusElement *countBarSymsBetween(int firstXpos, int actualXpos, int *count_of_bar_syms);
@@ -283,11 +286,12 @@ class NVoice {
 		void appendNoteAt(int line, int offs, unsigned int status);
 		void appendElem(int el_type, int line, int sub_type, int offs = 0, unsigned int status = 0);
 		void appendElem(NMusElement *elem);
-		void correctReadTrillsSlursAndDynamicsAndVAs();
+		void correctReadTrillsSlursAndDynamicsStringsAndVAs();
 		bool setProvisionalTrill(int kind, unsigned int at, unsigned int till_meascount, unsigned int till, NMusElement *last_bar_sym);
 		bool setProvisionalDynamic(int kind, unsigned int at, unsigned int till_meascount, unsigned int till, NMusElement *last_bar_sym);
 		bool setProvisionalSlur(unsigned int at, unsigned int till_meascount, unsigned int till, NMusElement *last_bar_sym);
 		bool setProvisionalOctaviation(int kind, unsigned int at, unsigned int till_meascount, unsigned int till, NMusElement *last_bar_sym);
+		void setProvisionalString(char *text, int type, unsigned int at, NMusElement *last_bar_sym);
 		bool setReadArpeggio(unsigned int at, NMusElement *last_bar_sym);
 		bool setReadPedalOn(unsigned int at, NMusElement *last_bar_sym);
 		bool setReadPedalOff(unsigned int at, NMusElement *last_bar_sym);
@@ -303,6 +307,7 @@ class NVoice {
 		void addLyrics(char *charlyrics, int verse);
 		void copyLyricsToEditor();
 		void collChords();
+		QList<NText> provStrings_;
 /*------------------------------ Lyrics -------------------------------------------*/
 		void updateLyrics();
 		int countOfLyricsLines();
